import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.util.*;
import java.io.*;

import org.jdom.Element;
import org.jdom.Document;
import org.jdom.output.XMLOutputter;

/**
 * This class is the base window for editing the map. It handles the
 * main menu operation and zoom in/out button from the toolbar. 
 *
 * Handling of the other toolbar buttons (graphic objects add, and
 * 'done', 'cancel' buttons) are done by the MapCanvas area.
 *
 * @author Alberto M. B. Simes
 */
public class MapViewer extends JFrame implements ActionListener,
						 PropertyChangeListener{
    private Map map;
    private JScrollPane pane;
    private MapCanvas area;
    private JLabel status;
    private String currentOp;

    private Layer currentLayer;
    private GeographicElement currentElement;

    private Hashtable operation;

    /**
     * The constructor receives a map object with the information to
     * be shown. All map changes will be reflected in this object.
     *
     * @param map Map object
     */
    public MapViewer(Map map) {
	super("Editing map to [" + map.getFile().getName() + "]");

	this.map = map;
	operation = new Hashtable();

	// infoBox
	Box infoBox = new Box(1);
	{
	    JList l;
	    infoBox.add(new JLabel("Layers"));

	    // Create a list for the layers
	    l = new JList();
	    operation.put("layers",l);
	    l.setVisibleRowCount(6);
	    infoBox.add(l);
	    l.addListSelectionListener( 
		new ListSelectionListener() {
		    public void valueChanged(ListSelectionEvent e) {
			
			JMenuItem item = (JMenuItem) operation.get("newElement");
			JList l = (JList)e.getSource();
			JList elements = (JList)operation.get("elements");
			
			if (l.getSelectedIndex() > -1) {
			    Object o = l.getSelectedValue();
			    currentLayer = (Layer)o;
			    status.setText("Selected layer '" + o.toString() + "'");
			    
			    elements.setListData(currentLayer.getElements());

			    item.setEnabled(true);
			} else {
			    item.setEnabled(false);
			    currentLayer = null;
			    status.setText("Ready");

			    elements.setListData(new Vector());
			}
			currentElement = null;
		    }
		});


	    infoBox.add(new JLabel("Elements"));

	    l = new JList();
	    operation.put("elements",l);
	    l.setVisibleRowCount(6);
	    infoBox.add(l);
	    l.addListSelectionListener( 
		new ListSelectionListener() {
		    public void valueChanged(ListSelectionEvent e) {
			JList l = (JList)e.getSource();
			if (l.getSelectedIndex() > -1) { 
			    Object o = l.getSelectedValue();
			    status.setText("Selected element '" + o.toString() + 
					   "' from layer '" + currentLayer.toString() + "'");
			    currentElement = (GeographicElement)o;
			    area.setEditable(true);
			    changeAddButtons(true);
			} else {
			    changeAddButtons(false);
			}
		    }
		});
	    infoBox.add(infoBox.createGlue());
	}

	// Create the mapCanvas and the scrollPane
	area = new MapCanvas(map);
	pane = new JScrollPane();
	pane.setViewportView(area);
	area.addPropertyChangeListener(this);

	//----------------------------------------------
	// Status bar
	//----------------------------------------------
	Box statusBox = new Box(0);
	statusBox.add(status = new JLabel("Ready"));
	statusBox.add(statusBox.createGlue());

	//----------------------------------------------
	// Create Toolbar
	//----------------------------------------------
	Box toolbarBox = new Box(0);
	JToolBar toolbar = new JToolBar();
	toolbarBox.add(toolbar);
	toolbarBox.add(toolbarBox.createGlue());
	{
	    JButton bt = new JButton(" Cancel ");
	    operation.put("Cancel", bt);
	    bt.setEnabled(false);
	    bt.addActionListener(this);
	    toolbar.add(bt);

	    bt = new JButton(" Done ");
	    operation.put("Done", bt);
	    bt.setEnabled(false);
	    bt.addActionListener(this);
	    toolbar.add(bt);

	    bt = new JButton(" Zoom In ");
	    operation.put("Zoom", bt);
	    bt.setEnabled(true);
	    bt.addActionListener(area);
	    toolbar.add(bt);

	    toolbar.addSeparator();

	    bt = new JButton(" Add Dot ");
	    operation.put("AddDot", bt);
	    bt.setEnabled(false);
	    bt.addActionListener(this);
	    toolbar.add(bt);

	    bt = new JButton(" Add Area ");
	    operation.put("AddArea", bt);
	    bt.setEnabled(false);
	    bt.addActionListener(this);
	    toolbar.add(bt);

	    bt = new JButton(" Add Path ");
	    operation.put("AddPath", bt);
	    bt.setEnabled(false);
	    bt.addActionListener(this);
	    toolbar.add(bt);

	    bt = new JButton(" Add Line ");
	    operation.put("AddLine", bt);
	    bt.setEnabled(false);
	    bt.addActionListener(this);
	    toolbar.add(bt);
	}

	// Create Vertical main box
	Box main;
	{
	    main = new Box(1);
	    main.add(toolbarBox);
	    Box hory = new Box(0);
	    {
		hory.add(pane);
		hory.add(infoBox);
	    }
	    main.add(hory);
	    main.add(statusBox);
	}

	JMenuBar menubar = new JMenuBar();
	JMenu menu;
	JMenuItem item;
	// menu FILE
	{
	    menu = new JMenu("File");

	    item = menu.add("Save");
	    menubar.add(menu);
	    operation.put("save", item);
	    item.addActionListener(this);

	    item = menu.add("Exit");
	    menubar.add(menu);
	}

	// menu NEW
	{
	    menu = new JMenu("New");

	    // new layer op
	    item = menu.add("Layer");
	    operation.put("newLayer", item);
	    item.addActionListener(this);

	    // new element op
	    item = menu.add("Element");
	    operation.put("newElement", item);
	    item.addActionListener(this);
	    item.setEnabled(false);

	    // new object op
	    item = menu.add("Object");
	    operation.put("newObject", item);
	    item.addActionListener(this);
	    item.setEnabled(false);

	    menubar.add(menu);
	}
	this.setJMenuBar(menubar);

	this.getContentPane().add(main);
	this.setSize(500,400);
    }

    /**
     * Implements action listener for some buttons
     *
     * @param event The button event
     */
    public void actionPerformed(ActionEvent event) {
	if (event.getActionCommand().equals("Save")) {
	    JFileChooser chooser = new JFileChooser();
	    chooser.addActionListener(this);
	    chooser.setFileFilter(new XmlFilter());
	    chooser.showSaveDialog(null);
	    return;
	}
	if (event.getActionCommand().equals("CancelSelection")) {
	    // DO NOTHING
	    return;
	}
	if (event.getActionCommand().equals("ApproveSelection")) {
	    JFileChooser chooser = (JFileChooser)event.getSource();
	    File save_file = chooser.getSelectedFile();

	    saveXML(save_file, map.toXML());

	    Message ok = new Message(this, "File saved!");
	    ok.show();
	    return;
	}
	// If the newLayer option was choosen
	if (event.getActionCommand().equals("Layer")) {
	    MapAddLayer dialog;
	    String layerName;
	    dialog = new MapAddLayer(this,map);
	    dialog.show();
	    layerName = dialog.getLayerName();
	    if (! layerName.equals("") ) {
		JList l = (JList)operation.get("layers");
		map.addLayer(layerName);
		l.setListData(map.getLayers());
	    }
	    return;
	}
	if (event.getActionCommand().equals("Element")) {
	    LayerAddElement dialog;
	    String elementName;
	    dialog = new LayerAddElement(this, map, currentLayer);
	    dialog.show();
	    elementName = dialog.getElementName();
	    if (! elementName.equals("") ) {
		JList l = (JList)operation.get("elements");
		currentLayer.addElement(elementName);
		l.setListData(currentLayer.getElements());
	    }
	    return;
	}
	if (event.getActionCommand().equals(" Cancel ")) {
	    area.setCurrentObject(null);
	    changeAddButtons(true);
	    area.repaint();
	    return;
	}
	if (event.getActionCommand().equals(" Done ")) {
	    GraphicElement ge = area.getCurrentObject();
	    ge.setActive(false);
	    currentElement.add(ge);
	    area.setCurrentObject(null);
	    changeAddButtons(true);
	    area.repaint();
	    return;
	}
	if (event.getActionCommand().equals(" Add Dot ")) {
	    GraphicElement el = new GraphicDot();
	    area.setCurrentObject(el);
	    changeAddButtons(false);
	    return;
	}		 
	if (event.getActionCommand().equals(" Add Area ")) {
	    GraphicElement el = new GraphicPolygon();
	    area.setCurrentObject(el);
	    changeAddButtons(false);
	    return;
	}
	if (event.getActionCommand().equals(" Add Line ")) {
	    GraphicElement el = new GraphicLine();
	    area.setCurrentObject(el);
	    changeAddButtons(false);
	    return;
	}
	System.out.println(event.toString());
    }

    /**
     * Implements property change event for mapCanvas object
     *
     * @param event The property change event
     */
    public void propertyChange(PropertyChangeEvent event) {
	if (event.getPropertyName().equals("zoom")) {
	    Boolean zoomed = (Boolean)event.getNewValue();
	    JViewport view = pane.getViewport();
	    Rectangle r = view.getViewRect();
	    if (zoomed.booleanValue()) {
		r.setLocation((int)((r.getX()+r.getWidth()/2)*2-r.getWidth()/2),
			      (int)((r.getY()+r.getHeight()/2)*2-r.getHeight()/2));
	    } else {
		r.setLocation((int)((r.getX()+r.getWidth()/2)/2-r.getWidth()/2),
			      (int)((r.getY()+r.getHeight()/2)/2-r.getHeight()/2));
	    }
	    view.setViewPosition(new Point((int)r.getX(), (int)r.getY()));
	}
    }

    /**
     *
     */
    private void changeAddButtons(boolean b) {
	((JButton)operation.get("AddDot")).setEnabled(b);
	((JButton)operation.get("AddArea")).setEnabled(b);
	((JButton)operation.get("AddPath")).setEnabled(b);
	((JButton)operation.get("AddLine")).setEnabled(b);

	((JButton)operation.get("Done")).setEnabled(!b);
	((JButton)operation.get("Cancel")).setEnabled(!b);
    }

    private void saveXML(File f, Element xml) {
	Document xml_document = new Document(xml);
	try {
	    XMLOutputter outputter = new XMLOutputter("  ", true);
	    FileWriter writer = new FileWriter(f);
	    outputter.output(xml_document, writer);
	    writer.close();
	} catch (java.io.IOException e) {
	    System.out.println("Error saving file: ");
	    e.printStackTrace();
	}
    }
    
}
